import { Component, OnInit, OnDestroy, ViewChild, ElementRef, ChangeDetectorRef } from '@angular/core';

import { Observable, Subscription, of } from 'rxjs';
import { catchError, exhaustMap, map, tap, take } from 'rxjs/operators';

import { ProbeService, requesterID } from '../../../commons/service/probe.service';

import { Interface } from '@overflow/model/net/nic';
import { Zone, DiscoverHost, Host, Port, Service } from '@overflow/model/discovery';
import { toMetaIPType, MetaIPTypeEnum } from '@overflow/model/meta';
import { Address } from '../../../commons/component/nic-dropdown.component';
import { RPCSubscriber } from '@overflow/commons/ui/decorator/RPCSubscriber';

const vis = require('vis');

@Component({
  selector: 'app-pages-home',
  templateUrl: './home-page.component.html',
  styleUrls: ['./home-page.component.scss'],
})
export class HomePageComponent implements OnInit, OnDestroy {
  private discoveryTargetRef: ElementRef;

  @ViewChild('discoveryTarget') set discoveryTarget(content: ElementRef) {
    this.discoveryTargetRef = content;
  }

  blockedPanel = false;

  blockedDocument = false;
  addresses: Address[];

  showIntro = true;

  private visNetwork: any;
  private nodeSet: any;
  private edgeSet: any;

  constructor(
    private changeDetector: ChangeDetectorRef,
    private probeService: ProbeService,
  ) {
  }

  ngOnInit() {
    this.probeService.subscribeNotification(this);
  }

  ngOnDestroy(): void {
    this.probeService.unsubscribeNotification(this);
  }

  blockDocument() {
    this.blockedDocument = true;
    setTimeout(() => {
      this.blockedDocument = false;
    }, 3000);
  }

  discoverHost(dfd: string) {
    this.showIntro = false;
    this.changeDetector.detectChanges();

    this.nodeSet = new vis.DataSet([
      { id: '192.168.1.0/24', label: 'Zone' },
    ]);
    this.edgeSet = new vis.DataSet([]);

    const options = {
      width: '100%',
      height: '100%'
    };

    this.visNetwork = new vis.Network(this.discoveryTargetRef.nativeElement, { nodes: this.nodeSet, edges: this.edgeSet }, options);


    const zone: Zone = {
      network: '192.168.1.0/24',
      iface: 'enp3s0',
      metaIPType: toMetaIPType(MetaIPTypeEnum.V4),
      address: '192.168.1.101',
      mac: '44:8a:5b:f1:f1:f3',
    };

    const discoverHost: DiscoverHost = {
      metaIPType: toMetaIPType(MetaIPTypeEnum.V4),
      firstScanRange: '192.168.1.1',
      lastScanRange: '192.168.1.254',
      discoveryConfig: {
      },
      discoverPort: {
        firstScanRange: 1,
        lastScanRange: 65535,
        includeTCP: true,
        includeUDP: true,
        discoverService: {
        }
      }
    };

    this.probeService.send('DiscoveryService.DiscoverHost', requesterID, zone, discoverHost);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.Queueing' })
  public DiscoveryQueueing(queueingDate: Date) {
    console.log('DiscoveryQueueing', queueingDate);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.QueueingFailed' })
  public DiscoveryQueueingFailed(queueingFailedDate: Date) {
    console.log('DiscoveryQueueingFailed', queueingFailedDate);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.QueueingTimeout' })
  public DiscoveryQueueingTimeout(queueingTimeoutDate: Date) {
    console.log('DiscoveryQueueingTimeout', queueingTimeoutDate);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.DiscoveryStart' })
  public DiscoveryStart(startDate: Date) {
    console.log('DiscoveryStart', startDate);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.DiscoveryStop' })
  public DiscoveryStop(stopDate: Date) {
    console.log('DiscoveryStop', stopDate);
    this.visNetwork.stabilize();
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.DiscoveryError' })
  public DiscoveryError(err: Error) {
    console.log('DiscoveryError', err);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.DiscoveredHost' })
  public DiscoveredHost(host: Host) {
    console.log('DiscoveredHost', host);
    const hostId = `${host.address}`;

    if (null !== this.nodeSet.get(hostId)) {
      this.nodeSet.update([
        { id: hostId, label: `${host.name}(${host.address})` }
      ]);
    } else {
      this.nodeSet.add([
        { id: hostId, label: `${host.name}(${host.address})` }
      ]);
      this.edgeSet.add([
        { from: '192.168.1.0/24', to: hostId },
      ]);
    }

  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.DiscoveredPort' })
  public DiscoveredPort(port: Port) {
    console.log('DiscoveredPort', port);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.DiscoveredService' })
  public DiscoveredService(service: Service) {
    console.log('DiscoveredService', service);
    const hostId = service.port.host.address;
    const serviceId = `${service.port.host.address}-${service.port.portNumber}-${service.key}`;
    this.nodeSet.add([
      { id: `${serviceId}`, label: `${service.key}` }
    ]);
    this.edgeSet.add([
      { from: `${hostId}`, to: `${serviceId}` },
    ]);

  }
}
